@@ -8,16 +8,17 @@ use v5.10;
 
 use Moo 1.000000;
 
+use Devel::StrictMode;
 use IO::Socket 1.18 ();
-use IO::String;
 use MooX::TypeTiny;
+use Scalar::Util qw/ refaddr /;
 use Sub::Quote qw/ quote_sub /;
 use Sub::Util 1.40 qw/ set_subname /;
 use Net::Statsd::Lite::Types -types;
 
 use namespace::autoclean;
 
-our $VERSION = 'v0.3.1';
+our $VERSION = 'v0.4.0';
 
 
 has host => (
@@ -54,13 +55,7 @@ has autoflush => (
     default => 1,
 );
 
-has _buffer => (
-    is      => 'lazy',
-    isa     => InstanceOf ['IO::String'],
-    builder => sub {
-        IO::String->new;
-    },
-);
+my %Buffers;
 
 
 has max_buffer_size => (
@@ -89,18 +84,16 @@ BEGIN {
     my $class = __PACKAGE__;
 
     my %PROTOCOL = (
-        set_add   => [ 's',  Str, ],
-        counter   => [ 'c',  Int, 1 ],
-        gauge     => [ 'g',  Gauge | PosInt ],
-        histogram => [ 'h',  PosNum ],
-        meter     => [ 'm',  PosInt ],
-        timing    => [ 'ms', PosNum, 1 ],
+        set_add   => [ '|s',  Str, ],
+        counter   => [ '|c',  Int, 1 ],
+        gauge     => [ '|g',  Gauge | PosInt ],
+        histogram => [ '|h',  PosNum ],
+        meter     => [ '|m',  PosInt ],
+        timing    => [ '|ms', PosNum, 1 ],
     );
 
     foreach my $name ( keys %PROTOCOL ) {
 
-        no strict 'refs'; ## no critic (ProhibitNoStrict)
-
         my $type = $PROTOCOL{$name}[1];
         my $rate = $PROTOCOL{$name}[2];
 
@@ -109,17 +102,22 @@ BEGIN {
           ? q{ my ($self, $metric, $value, $rate) = @_; }
           : q{ my ($self, $metric, $value) = @_; };
 
-        $code .= $type->inline_assert('$value');
+        if (STRICT) {
 
-        $code .= q/ if (defined $rate) { / . Rate->inline_assert('$rate') . ' }'
-          if defined $rate;
+            $code .= $type->inline_assert('$value');
 
-        my $tmpl = '%s:%s|' . $PROTOCOL{$name}[0];
+            $code .=
+              q/ if (defined $rate) { / . Rate->inline_assert('$rate') . ' }'
+              if defined $rate;
+        }
+
+        my $tmpl  = $PROTOCOL{$name}[0];
 
         if ( defined $rate ) {
 
             $code .= q/ if ((defined $rate) && ($rate<1)) {
-                     $self->_record( $tmpl . '|@%f', $metric, $value, $rate );
+                     $self->_record( $tmpl . '|@' . $rate, $metric, $value )
+                        if rand() <= $rate;
                    } else {
                      $self->_record( $tmpl, $metric, $value ); } /;
         }
@@ -130,20 +128,18 @@ BEGIN {
         }
 
         quote_sub "${class}::${name}", $code,
-          { '$tmpl' => \$tmpl },
+          { '$tmpl'  => \$tmpl },
           { no_defer => 1 };
 
-
     }
 
     # Alises for other Net::Statsd::Client or Etsy::StatsD
 
     {
-        no strict 'refs'; ## no critic (ProhibitNoStrict)
+        no strict 'refs';    ## no critic (ProhibitNoStrict)
 
         *{"${class}::update"}    = set_subname "update"    => \&counter;
         *{"${class}::timing_ms"} = set_subname "timing_ms" => \&timing;
-        *{"${class}::add_set"}   = set_subname "timing_ms" => \&set_add;
 
     }
 
@@ -160,42 +156,41 @@ sub decrement {
 }
 
 sub _record {
-    my ( $self, $template, @args ) = @_;
+    my ( $self, $suffix, $metric, $value ) = @_;
 
-    my $data = $self->prefix . sprintf( $template, @args );
+    my $data = $self->prefix . $metric . ':' . $value . $suffix . "\n";
 
-    my $fh  = $self->_buffer;
-    my $len = length($data);
-
-    if ( $len >= $self->max_buffer_size ) {
-        warn "Data is too large";
-        return $self;
+    if ( $self->autoflush ) {
+        send( $self->_socket, $data, 0 );
+        return;
     }
 
-    $len += length( ${ $fh->string_ref } );
-    if ( $len >= $self->max_buffer_size ) {
-        $self->flush;
-    }
+    my $index = refaddr $self;
+    my $avail = $self->max_buffer_size - length( $Buffers{$index} );
 
-    say {$fh} $data;
+    $self->flush if length($data) > $avail;
+
+    $Buffers{$index} .= $data;
 
-    $self->flush if $self->autoflush;
 }
 
 
 sub flush {
     my ($self) = @_;
 
-    my $fh = $self->_buffer;
-
-    my $data = ${ $fh->string_ref };
-
-    if ( length($data) ) {
-        $self->_send( $data, 0 );
-        $fh->truncate;
+    my $index = refaddr $self;
+    if ( $Buffers{$index} ne '' ) {
+        send( $self->_socket, $Buffers{$index}, 0 );
+        $Buffers{$index} = '';
     }
 }
 
+sub BUILD {
+    my ($self) = @_;
+
+    $Buffers{ refaddr $self } = '';
+}
+
 sub DEMOLISH {
     my ( $self, $is_global ) = @_;
 
@@ -219,7 +214,7 @@ Net::Statsd::Lite - A lightweight StatsD client that supports multimetric packet
 
 =head1 VERSION
 
-version v0.3.1
+version v0.4.0
 
 =head1 SYNOPSIS
 
@@ -390,16 +385,18 @@ specification recommends that integers be used.
 This adds the the C<$string> to a set, for logging the number of
 unique things, e.g. IP addresses or usernames.
 
-=head2 C<add_set>
-
-This is an alias for L</set_add>. It is deprecated and will be removed
-in a later version.
-
 =head2 C<flush>
 
 This sends the buffer to the L</host> and empties the buffer, if there
 is any data in the buffer.
 
+=head1 STRICT MODE
+
+If this module is first loaded in C<STRICT> mode, then the values and
+rate arguments will be checked that thet are the correct type.
+
+See L<Devel::StrictMode> for more information.
+
 =head1 SEE ALSO
 
 This module was forked from L<Net::Statsd::Tiny>.
